package dev.peerat.parser.java.builder;

import java.util.ArrayList;
import java.util.List;

import dev.peerat.parser.Token;
import dev.peerat.parser.java.Annotation;
import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.java.Variable;

public class JavaFunctionBuilder extends JavaModifiableBuilder<JavaFunctionBuilder, Function>{

	private List<Annotation> annotations;
	private String generic;
	private String type;
	private String name;
	private List<Variable> parameters;
	private List<Token> throwables;
	
	private List<JavaElement> elements;
	
	public JavaFunctionBuilder annotate(JavaAnnotationBuilder annotation){
		if(this.annotations == null) this.annotations = new ArrayList<>();
		this.annotations.add(annotation.build());
		return this;
	}
	
	public JavaFunctionBuilder setGeneric(String generic){
		this.generic = generic;
		return this;
	}
	
	public JavaFunctionBuilder setType(String type){
		this.type = type;
		return this;
	}
	
	public JavaFunctionBuilder setName(String name){
		this.name = name;
		return this;
	}
	
	public JavaFunctionBuilder parameter(JavaVariableBuilder builder){
		if(this.parameters == null) this.parameters = new ArrayList<>();
		this.parameters.add(builder.build());
		return this;
	}
	
	public JavaFunctionBuilder throwable(String throwable){
		if(this.throwables == null) this.throwables = new ArrayList<>();
		this.throwables.add(new Token(0,0,throwable, null));
		return this;
	}
	
	public JavaFunctionBuilder element(JavaElement element){
		if(this.elements == null) this.elements = new ArrayList<>();
		this.elements.add(element);
		return this;
	}
	
	@Override
	public Function build(){
		Function result = new Function(
				this.annotations,
				this.mod,
				new Token(0,0, this.generic, null),
				new Token(0,0, this.type, null),
				new Token(0,0, this.name, null),
				this.parameters,
				this.throwables
				);
		result.getElements().addAll(elements);
		return result;
	}
	
	

}
